#ifndef QIICRUIDs_h
#define QIICRUIDs_h

#define QIICR_UID_ROOT "1.3.6.1.4.1.43046.3"
#define QIICR_IMPLEMENTATION_CLASS_UID QIICR_UID_ROOT ".0.1"
#define QIICR_CODING_SCHEME_UID_ROOT QIICR_UID_ROOT ".0.0"

#define QIICR_DEVICE_OBSERVER_UID QIICR_IMPLEMENTATION_CLASS_UID ".99.1" // use .99 for sandbox code examples

#define QIICR_INSTANCE_UID_ROOT QIICR_UID_ROOT ".1.4"

#endif
